/*
 * JBoss, Home of Professional Open Source
 * Copyright 2011 Red Hat Inc. and/or its affiliates and other contributors
 * as indicated by the @author tags. All rights reserved.
 * See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU Lesser General Public License, v. 2.1.
 * This program is distributed in the hope that it will be useful, but WITHOUT A
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License,
 * v.2.1 along with this distribution; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 */

package org.jboss.as.console.client.core.settings;

import com.google.gwt.event.shared.EventBus;
import com.google.inject.Inject;
import com.gwtplatform.mvp.client.Presenter;
import com.gwtplatform.mvp.client.View;
import com.gwtplatform.mvp.client.annotations.NameToken;
import com.gwtplatform.mvp.client.annotations.ProxyCodeSplit;
import com.gwtplatform.mvp.client.proxy.Place;
import com.gwtplatform.mvp.client.proxy.PlaceManager;
import com.gwtplatform.mvp.client.proxy.Proxy;
import com.gwtplatform.mvp.client.proxy.RevealRootPopupContentEvent;
import org.jboss.as.console.client.core.NameTokens;

/**
 * @author Heiko Braun
 * @date 5/3/11
 */
public class SettingsPresenter extends Presenter<SettingsPresenter.MyView, SettingsPresenter.MyProxy> {

    private final PlaceManager placeManager;
    private SettingsPresenterWidget settingsWidget;

    @ProxyCodeSplit
    @NameToken(NameTokens.SettingsPresenter)
    public interface MyProxy extends Proxy<SettingsPresenter>, Place {
    }

    public interface MyView extends View {
        void setPresenter(SettingsPresenter presenter);
    }

    @Inject
    public SettingsPresenter(
            EventBus eventBus, MyView view, MyProxy proxy,
            PlaceManager placeManager,
            SettingsPresenterWidget settingsWidget) {
        super(eventBus, view, proxy);

        this.placeManager = placeManager;
        this.settingsWidget = settingsWidget;


    }

    @Override
    protected void onBind() {
        super.onBind();
        getView().setPresenter(this);
    }

    @Override
    protected void onReveal() {
        super.onReveal();
        System.out.println("reveal");
    }

    @Override
    protected void onReset() {
        super.onReset();
        System.out.println("reset");
    }

    @Override
    protected void revealInParent() {

        RevealRootPopupContentEvent.fire(this, settingsWidget);
        //addToPopupSlot(settingsWidget, true);


    }

}
